@@ -3,6 +3,8 @@
    RDP licensing negotiation
    Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 1999-2008
    Copyright (C) Thomas Uhle <thomas.uhle@mailbox.tu-dresden.de> 2011
+   Copyright (C) Henrik Andersson <henrik.andersson@cendio.com> 2014
+
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -23,12 +25,13 @@
 
 extern char *g_username;
 extern char g_hostname[16];
-extern RD_BOOL g_use_rdp5;
+extern RDP_VERSION g_rdp_version;
 
 static uint8 g_licence_key[16];
 static uint8 g_licence_sign_key[16];
 
 RD_BOOL g_licence_issued = False;
+RD_BOOL g_licence_error_result = False;
 
 /* Generate a session key and RC4 keys, given client and server randoms */
 static void
@@ -55,10 +58,10 @@ licence_generate_hwid(uint8 * hwid)
 	strncpy((char *) (hwid + 4), g_hostname, LICENCE_HWID_SIZE - 4);
 }
 
-/* Present an existing licence to the server */
+/* Send a lincece info packet to server */
 static void
-licence_present(uint8 * client_random, uint8 * rsa_data,
-		uint8 * licence_data, int licence_size, uint8 * hwid, uint8 * signature)
+licence_info(uint8 * client_random, uint8 * rsa_data,
+	     uint8 * licence_data, int licence_size, uint8 * hwid, uint8 * signature)
 {
 	uint32 sec_flags = SEC_LICENCE_NEG;
 	uint16 length =
@@ -68,8 +71,8 @@ licence_present(uint8 * client_random, uint8 * rsa_data,
 
 	s = sec_init(sec_flags, length + 2);
 
-	out_uint8(s, LICENCE_TAG_PRESENT);
-	out_uint8(s, (g_use_rdp5? 3 : 2));	/* version */
+	out_uint8(s, LICENCE_TAG_LICENCE_INFO);
+	out_uint8(s, ((g_rdp_version >= RDP_V5) ? 3 : 2));	/* version */
 	out_uint16_le(s, length);
 
 	out_uint32_le(s, 1);
@@ -96,25 +99,24 @@ licence_present(uint8 * client_random, uint8 * rsa_data,
 	sec_send(s, sec_flags);
 }
 
-/* Send a licence request packet */
+/* Send a new licence request packet */
 static void
-licence_send_request(uint8 * client_random, uint8 * rsa_data, char *user, char *host)
+licence_send_new_licence_request(uint8 * client_random, uint8 * rsa_data, char *user, char *host)
 {
 	uint32 sec_flags = SEC_LICENCE_NEG;
 	uint16 userlen = strlen(user) + 1;
 	uint16 hostlen = strlen(host) + 1;
-	uint16 length = 
-		24 + SEC_RANDOM_SIZE + SEC_MODULUS_SIZE + SEC_PADDING_SIZE + 
-		userlen + hostlen;
+	uint16 length =
+		24 + SEC_RANDOM_SIZE + SEC_MODULUS_SIZE + SEC_PADDING_SIZE + userlen + hostlen;
 	STREAM s;
 
 	s = sec_init(sec_flags, length + 2);
 
-	out_uint8(s, LICENCE_TAG_REQUEST);
-	out_uint8(s, (g_use_rdp5? 3 : 2));	/* version */
+	out_uint8(s, LICENCE_TAG_NEW_LICENCE_REQUEST);
+	out_uint8(s, ((g_rdp_version >= RDP_V5) ? 3 : 2));	/* version */
 	out_uint16_le(s, length);
 
-	out_uint32_le(s, 1);
+	out_uint32_le(s, 1);	// KEY_EXCHANGE_ALG_RSA
 	out_uint16(s, 0);
 	out_uint16_le(s, 0xff01);
 
@@ -124,11 +126,13 @@ licence_send_request(uint8 * client_random, uint8 * rsa_data, char *user, char *
 	out_uint8p(s, rsa_data, SEC_MODULUS_SIZE);
 	out_uint8s(s, SEC_PADDING_SIZE);
 
-	out_uint16_le(s, LICENCE_TAG_USER);
+	/* Username LICENSE_BINARY_BLOB */
+	out_uint16_le(s, BB_CLIENT_USER_NAME_BLOB);
 	out_uint16_le(s, userlen);
 	out_uint8p(s, user, userlen);
 
-	out_uint16_le(s, LICENCE_TAG_HOST);
+	/* Machinename LICENSE_BINARY_BLOB */
+	out_uint16_le(s, BB_CLIENT_MACHINE_NAME_BLOB);
 	out_uint16_le(s, hostlen);
 	out_uint8p(s, host, hostlen);
 
@@ -136,9 +140,9 @@ licence_send_request(uint8 * client_random, uint8 * rsa_data, char *user, char *
 	sec_send(s, sec_flags);
 }
 
-/* Process a licence demand packet */
+/* Process a licence request packet */
 static void
-licence_process_demand(STREAM s)
+licence_process_request(STREAM s)
 {
 	uint8 null_data[SEC_MODULUS_SIZE];
 	uint8 *server_random;
@@ -146,7 +150,7 @@ licence_process_demand(STREAM s)
 	uint8 hwid[LICENCE_HWID_SIZE];
 	uint8 *licence_data;
 	int licence_size;
-	SSL_RC4 crypt_key;
+	RDSSL_RC4 crypt_key;
 
 	/* Retrieve the server random from the incoming packet */
 	in_uint8p(s, server_random, SEC_RANDOM_SIZE);
@@ -164,27 +168,27 @@ licence_process_demand(STREAM s)
 		sec_sign(signature, 16, g_licence_sign_key, 16, hwid, sizeof(hwid));
 
 		/* Now encrypt the HWID */
-		ssl_rc4_set_key(&crypt_key, g_licence_key, 16);
-		ssl_rc4_crypt(&crypt_key, hwid, hwid, sizeof(hwid));
+		rdssl_rc4_set_key(&crypt_key, g_licence_key, 16);
+		rdssl_rc4_crypt(&crypt_key, hwid, hwid, sizeof(hwid));
 
 #if WITH_DEBUG
-		DEBUG(("Sending licensing PDU (message type 0x%02x)\n", LICENCE_TAG_PRESENT));
+		DEBUG(("Sending licensing PDU (message type 0x%02x)\n", LICENCE_TAG_LICENCE_INFO));
 #endif
-		licence_present(null_data, null_data, licence_data, licence_size, hwid, signature);
+		licence_info(null_data, null_data, licence_data, licence_size, hwid, signature);
 
 		xfree(licence_data);
 		return;
 	}
 
 #if WITH_DEBUG
-	DEBUG(("Sending licensing PDU (message type 0x%02x)\n", LICENCE_TAG_REQUEST));
+	DEBUG(("Sending licensing PDU (message type 0x%02x)\n", LICENCE_TAG_NEW_LICENCE_REQUEST));
 #endif
-	licence_send_request(null_data, null_data, g_username, g_hostname);
+	licence_send_new_licence_request(null_data, null_data, g_username, g_hostname);
 }
 
-/* Send an authentication response packet */
+/* Send a platform challange response packet */
 static void
-licence_send_authresp(uint8 * token, uint8 * crypt_hwid, uint8 * signature)
+licence_send_platform_challange_response(uint8 * token, uint8 * crypt_hwid, uint8 * signature)
 {
 	uint32 sec_flags = SEC_LICENCE_NEG;
 	uint16 length = 58;
@@ -192,8 +196,8 @@ licence_send_authresp(uint8 * token, uint8 * crypt_hwid, uint8 * signature)
 
 	s = sec_init(sec_flags, length + 2);
 
-	out_uint8(s, LICENCE_TAG_AUTHRESP);
-	out_uint8(s, (g_use_rdp5? 3 : 2));	/* version */
+	out_uint8(s, LICENCE_TAG_PLATFORM_CHALLANGE_RESPONSE);
+	out_uint8(s, ((g_rdp_version >= RDP_V5) ? 3 : 2));	/* version */
 	out_uint16_le(s, length);
 
 	out_uint16_le(s, 1);
@@ -210,9 +214,9 @@ licence_send_authresp(uint8 * token, uint8 * crypt_hwid, uint8 * signature)
 	sec_send(s, sec_flags);
 }
 
-/* Parse an authentication request packet */
+/* Parse an platform challange request packet */
 static RD_BOOL
-licence_parse_authreq(STREAM s, uint8 ** token, uint8 ** signature)
+licence_parse_platform_challange(STREAM s, uint8 ** token, uint8 ** signature)
 {
 	uint16 tokenlen;
 
@@ -231,24 +235,24 @@ licence_parse_authreq(STREAM s, uint8 ** token, uint8 ** signature)
 	return s_check_end(s);
 }
 
-/* Process an authentication request packet */
+/* Process a platform challange  packet */
 static void
-licence_process_authreq(STREAM s)
+licence_process_platform_challange(STREAM s)
 {
 	uint8 *in_token = NULL, *in_sig;
 	uint8 out_token[LICENCE_TOKEN_SIZE], decrypt_token[LICENCE_TOKEN_SIZE];
 	uint8 hwid[LICENCE_HWID_SIZE], crypt_hwid[LICENCE_HWID_SIZE];
 	uint8 sealed_buffer[LICENCE_TOKEN_SIZE + LICENCE_HWID_SIZE];
 	uint8 out_sig[LICENCE_SIGNATURE_SIZE];
-	SSL_RC4 crypt_key;
+	RDSSL_RC4 crypt_key;
 
 	/* Parse incoming packet and save the encrypted token */
-	licence_parse_authreq(s, &in_token, &in_sig);
+	licence_parse_platform_challange(s, &in_token, &in_sig);
 	memcpy(out_token, in_token, LICENCE_TOKEN_SIZE);
 
 	/* Decrypt the token. It should read TEST in Unicode. */
-	ssl_rc4_set_key(&crypt_key, g_licence_key, 16);
-	ssl_rc4_crypt(&crypt_key, in_token, decrypt_token, LICENCE_TOKEN_SIZE);
+	rdssl_rc4_set_key(&crypt_key, g_licence_key, 16);
+	rdssl_rc4_crypt(&crypt_key, in_token, decrypt_token, LICENCE_TOKEN_SIZE);
 
 	/* Generate a signature for a buffer of token and HWID */
 	licence_generate_hwid(hwid);
@@ -257,41 +261,33 @@ licence_process_authreq(STREAM s)
 	sec_sign(out_sig, 16, g_licence_sign_key, 16, sealed_buffer, sizeof(sealed_buffer));
 
 	/* Now encrypt the HWID */
-	ssl_rc4_set_key(&crypt_key, g_licence_key, 16);
-	ssl_rc4_crypt(&crypt_key, hwid, crypt_hwid, LICENCE_HWID_SIZE);
+	rdssl_rc4_set_key(&crypt_key, g_licence_key, 16);
+	rdssl_rc4_crypt(&crypt_key, hwid, crypt_hwid, LICENCE_HWID_SIZE);
 
-#if WITH_DEBUG
-	DEBUG(("Sending licensing PDU (message type 0x%02x)\n", LICENCE_TAG_AUTHRESP));
-#endif
-	licence_send_authresp(out_token, crypt_hwid, out_sig);
+	licence_send_platform_challange_response(out_token, crypt_hwid, out_sig);
 }
 
-/* Process an licence issue packet */
+/* Process a new licence packet */
 static void
-licence_process_issue(STREAM s)
+licence_process_new_license(STREAM s)
 {
-	SSL_RC4 crypt_key;
+	RDSSL_RC4 crypt_key;
 	uint32 length;
-	uint16 check;
 	int i;
 
-	in_uint8s(s, 2);	/* 3d 45 - unknown */
+	in_uint8s(s, 2);	// Skip license binary blob type
 	in_uint16_le(s, length);
 	if (!s_check_rem(s, length))
 		return;
 
-	ssl_rc4_set_key(&crypt_key, g_licence_key, 16);
-	ssl_rc4_crypt(&crypt_key, s->p, s->p, length);
+	rdssl_rc4_set_key(&crypt_key, g_licence_key, 16);
+	rdssl_rc4_crypt(&crypt_key, s->p, s->p, length);
 
-	in_uint16(s, check);
-	if (check != 0)
-		return;
+	/* Parse NEW_LICENSE_INFO block */
+	in_uint8s(s, 4);	// skip dwVersion
 
-	g_licence_issued = True;
-
-	in_uint8s(s, 2);	/* pad */
-
-	/* advance to fourth string */
+	/* Skip strings, Scope, CompanyName and ProductId to get
+	   to the LicenseInfo which we store in license blob. */
 	length = 0;
 	for (i = 0; i < 4; i++)
 	{
@@ -305,6 +301,53 @@ licence_process_issue(STREAM s)
 	save_licence(s->p, length);
 }
 
+/* process a licence error alert packet */
+void
+licence_process_error_alert(STREAM s)
+{
+	uint32 error_code;
+	uint32 state_transition;
+	uint32 error_info;
+	in_uint32(s, error_code);
+	in_uint32(s, state_transition);
+	in_uint32(s, error_info);
+
+	/* There is a special case in the error alert handling, when licensing is all good
+	   and the server is not sending a license to client, a "Server License Error PDU -
+	   Valid Client" packet is sent which means, every thing is ok.
+
+	   Therefor we should flag that everything is ok with license here.
+	 */
+	if (error_code == 0x07)
+	{
+		g_licence_issued = True;
+		return;
+	}
+
+	/* handle error codes, for now, jsut report them */
+	switch (error_code)
+	{
+		case 0x6:	// ERR_NO_LICENSE_SERVER
+			warning("License error alert from server: No license server\n");
+			break;
+
+		case 0x8:	// ERR_INVALID_CLIENT
+			warning("License error alert from server: Invalid client\n");
+			break;
+
+		case 0x4:	// ERR_INVALID_SCOPE
+		case 0xb:	// ERR_INVALID_PRODUCTID
+		case 0xc:	// ERR_INVALID_MESSAGE_LENGTH
+		default:
+			warning("License error alert from server: code %u, state transition %u\n",
+				error_code, state_transition);
+			break;
+	}
+
+	g_licence_error_result = True;
+}
+
+
 /* Process a licence packet */
 void
 licence_process(STREAM s)
@@ -320,20 +363,22 @@ licence_process(STREAM s)
 
 	switch (tag)
 	{
-		case LICENCE_TAG_DEMAND:
-			licence_process_demand(s);
+		case LICENCE_TAG_REQUEST:
+			licence_process_request(s);
 			break;
 
-		case LICENCE_TAG_AUTHREQ:
-			licence_process_authreq(s);
+		case LICENCE_TAG_PLATFORM_CHALLANGE:
+			licence_process_platform_challange(s);
 			break;
 
-		case LICENCE_TAG_ISSUE:
-		case LICENCE_TAG_REISSUE:
-			licence_process_issue(s);
+		case LICENCE_TAG_NEW_LICENCE:
+		case LICENCE_TAG_UPGRADE_LICENCE:
+			/* we can handle new and upgrades of licences the same way. */
+			licence_process_new_license(s);
 			break;
 
-		case LICENCE_TAG_RESULT:
+		case LICENCE_TAG_ERROR_ALERT:
+			licence_process_error_alert(s);
 			break;
 
 		default:
