@@ -189,6 +189,9 @@ rdp_send_data(STREAM s, uint8 data_pdu_type)
 void
 rdp_out_unistr(STREAM s, char *string, int len)
 {
+	if (string == NULL || len == 0)
+		return;
+
 #ifdef HAVE_ICONV
 	size_t ibl = strlen(string), obl = len + 2;
 	static iconv_t iconv_h = (iconv_t) - 1;
@@ -1733,14 +1736,26 @@ RD_BOOL
 rdp_connect(char *server, uint32 flags, char *domain, char *password,
 	    char *command, char *directory, RD_BOOL reconnect)
 {
+	RD_BOOL deactivated = False;
+	uint32 ext_disc_reason = 0;
+
 	if (!sec_connect(server, g_username, domain, password, reconnect))
 		return False;
 
 	rdp_send_logon_info(flags, domain, g_username, password, command, directory);
 
-	if (g_network_error)
-		return False;
+	/* run RDP loop until first licence demand active PDU */
+	while (!g_rdp_shareid)
+	{
+		if (g_network_error)
+			return False;
 
+		if (!rdp_loop(&deactivated, &ext_disc_reason))
+			return False;
+
+		if (g_redirect)
+			return True;
+	}
 	return True;
 }
 
