@@ -1,13 +1,13 @@
-/* 
+/* -*- c-basic-offset: 8 -*-
    rdesktop: A Remote Desktop Protocol client.
    Sound Channel Process Functions - SGI/IRIX
-   Copyright (C) Matthew Chapman 2003
-   Copyright (C) GuoJunBo guojunbo@ict.ac.cn 2003
-   Copyright (C) Jeremy Meng voidfoo@cwazy.co.uk 2004
+   Copyright (C) Matthew Chapman <matthewc.unsw.edu.au> 2003-2008
+   Copyright (C) GuoJunBo <guojunbo@ict.ac.cn> 2003
+   Copyright (C) Jeremy Meng <void.foo@gmail.com> 2004-2005
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -16,113 +16,119 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "rdesktop.h"
 #include <errno.h>
 #include <dmedia/audio.h>
 
-#define IRIX_DEBUG 1
+/* #define IRIX_DEBUG 1 */
 
 #define IRIX_MAX_VOL     65535
 
-#define MAX_QUEUE	10
-
-int g_dsp_fd;
 ALconfig audioconfig;
 ALport output_port;
 
-BOOL g_dsp_busy = False;
-static BOOL g_swapaudio;
 static int g_snd_rate;
-static BOOL g_swapaudio;
 static int width = AL_SAMPLE_16;
+static char *sgi_output_device = NULL;
 
 double min_volume, max_volume, volume_range;
 int resource, maxFillable;
 int combinedFrameSize;
 
-static struct audio_packet
-{
-	struct stream s;
-	uint16 tick;
-	uint8 index;
-} packet_queue[MAX_QUEUE];
-static unsigned int queue_hi, queue_lo;
+void sgi_play(void);
 
-BOOL
-wave_out_open(void)
+void
+sgi_add_fds(int *n, fd_set * rfds, fd_set * wfds, struct timeval *tv)
+{
+	/* We need to be called rather often... */
+	if (output_port != (ALport) 0 && !rdpsnd_queue_empty())
+		FD_SET(0, wfds);
+}
+
+void
+sgi_check_fds(fd_set * rfds, fd_set * wfds)
+{
+	if (output_port == (ALport) 0)
+		return;
+
+	if (!rdpsnd_queue_empty())
+		sgi_play();
+}
+
+RD_BOOL
+sgi_open(void)
 {
 	ALparamInfo pinfo;
+	static int warned = 0;
 
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_open: begin\n");
+	fprintf(stderr, "sgi_open: begin\n");
 #endif
 
+	if (!warned && sgi_output_device)
+	{
+		warning("device-options not supported for libao-driver\n");
+		warned = 1;
+	}
+
 	if (alGetParamInfo(AL_DEFAULT_OUTPUT, AL_GAIN, &pinfo) < 0)
 	{
-		fprintf(stderr, "wave_out_open: alGetParamInfo failed: %s\n",
+		fprintf(stderr, "sgi_open: alGetParamInfo failed: %s\n",
 			alGetErrorString(oserror()));
 	}
 	min_volume = alFixedToDouble(pinfo.min.ll);
 	max_volume = alFixedToDouble(pinfo.max.ll);
 	volume_range = (max_volume - min_volume);
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_open: minvol = %lf, maxvol= %lf, range = %lf.\n",
+	fprintf(stderr, "sgi_open: minvol = %lf, maxvol= %lf, range = %lf.\n",
 		min_volume, max_volume, volume_range);
 #endif
 
-	queue_lo = queue_hi = 0;
-
 	audioconfig = alNewConfig();
-	if (audioconfig < 0)
+	if (audioconfig == (ALconfig) 0)
 	{
-		fprintf(stderr, "wave_out_open: alNewConfig failed: %s\n",
-			alGetErrorString(oserror()));
+		fprintf(stderr, "sgi_open: alNewConfig failed: %s\n", alGetErrorString(oserror()));
 		return False;
 	}
 
 	output_port = alOpenPort("rdpsnd", "w", 0);
 	if (output_port == (ALport) 0)
 	{
-		fprintf(stderr, "wave_out_open: alOpenPort failed: %s\n",
-			alGetErrorString(oserror()));
+		fprintf(stderr, "sgi_open: alOpenPort failed: %s\n", alGetErrorString(oserror()));
 		return False;
 	}
 
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_open: returning\n");
+	fprintf(stderr, "sgi_open: returning\n");
 #endif
 	return True;
 }
 
 void
-wave_out_close(void)
+sgi_close(void)
 {
 	/* Ack all remaining packets */
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_close: begin\n");
+	fprintf(stderr, "sgi_close: begin\n");
 #endif
 
-	while (queue_lo != queue_hi)
-	{
-		rdpsnd_send_completion(packet_queue[queue_lo].tick, packet_queue[queue_lo].index);
-		free(packet_queue[queue_lo].s.data);
-		queue_lo = (queue_lo + 1) % MAX_QUEUE;
-	}
+	while (!rdpsnd_queue_empty())
+		rdpsnd_queue_next(0);
 	alDiscardFrames(output_port, 0);
 
 	alClosePort(output_port);
+	output_port = (ALport) 0;
 	alFreeConfig(audioconfig);
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_close: returning\n");
+	fprintf(stderr, "sgi_close: returning\n");
 #endif
 }
 
-BOOL
-wave_out_format_supported(WAVEFORMATEX * pwfx)
+RD_BOOL
+sgi_format_supported(RD_WAVEFORMATEX * pwfx)
 {
 	if (pwfx->wFormatTag != WAVE_FORMAT_PCM)
 		return False;
@@ -134,43 +140,34 @@ wave_out_format_supported(WAVEFORMATEX * pwfx)
 	return True;
 }
 
-BOOL
-wave_out_set_format(WAVEFORMATEX * pwfx)
+RD_BOOL
+sgi_set_format(RD_WAVEFORMATEX * pwfx)
 {
 	int channels;
 	int frameSize, channelCount;
 	ALpv params;
 
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_set_format: init...\n");
+	fprintf(stderr, "sgi_set_format: init...\n");
 #endif
 
-	g_swapaudio = False;
 	if (pwfx->wBitsPerSample == 8)
 		width = AL_SAMPLE_8;
 	else if (pwfx->wBitsPerSample == 16)
-	{
 		width = AL_SAMPLE_16;
-		/* Do we need to swap the 16bit values? (Are we BigEndian) */
-#if (defined(B_ENDIAN))
-		g_swapaudio = 1;
-#else
-		g_swapaudio = 0;
-#endif
-	}
 
-	/* Limited support to configure an opened audio port in IRIX The
-	 number of channels is a static setting and can not be changed after
-	 a port is opened. So if the number of channels remains the same, we
-	 can configure other settings Otherwise we have to reopen the audio
-	 port, using same config. */
+	/* Limited support to configure an opened audio port in IRIX.  The
+	   number of channels is a static setting and can not be changed after
+	   a port is opened.  So if the number of channels remains the same, we
+	   can configure other settings; otherwise we have to reopen the audio
+	   port, using same config. */
 
 	channels = pwfx->nChannels;
 	g_snd_rate = pwfx->nSamplesPerSec;
 
 	alSetSampFmt(audioconfig, AL_SAMPFMT_TWOSCOMP);
 	alSetWidth(audioconfig, width);
-	if ( channels != alGetChannels(audioconfig) )
+	if (channels != alGetChannels(audioconfig))
 	{
 		alClosePort(output_port);
 		alSetChannels(audioconfig, channels);
@@ -178,8 +175,8 @@ wave_out_set_format(WAVEFORMATEX * pwfx)
 
 		if (output_port == (ALport) 0)
 		{
-			fprintf(stderr, "wave_out_set_format: alOpenPort failed: %s\n",
-							alGetErrorString(oserror()));
+			fprintf(stderr, "sgi_set_format: alOpenPort failed: %s\n",
+				alGetErrorString(oserror()));
 			return False;
 		}
 
@@ -192,7 +189,7 @@ wave_out_set_format(WAVEFORMATEX * pwfx)
 
 	if (frameSize == 0 || channelCount == 0)
 	{
-		fprintf(stderr, "wave_out_set_format: bad frameSize or channelCount\n");
+		fprintf(stderr, "sgi_set_format: bad frameSize or channelCount\n");
 		return False;
 	}
 	combinedFrameSize = frameSize * channelCount;
@@ -213,20 +210,20 @@ wave_out_set_format(WAVEFORMATEX * pwfx)
 	}
 
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_set_format: returning...\n");
+	fprintf(stderr, "sgi_set_format: returning...\n");
 #endif
 	return True;
 }
 
 void
-wave_out_volume(uint16 left, uint16 right)
+sgi_volume(uint16 left, uint16 right)
 {
 	double gainleft, gainright;
 	ALpv pv[1];
 	ALfixed gain[8];
 
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_volume: begin\n");
+	fprintf(stderr, "sgi_volume: begin\n");
 	fprintf(stderr, "left='%d', right='%d'\n", left, right);
 #endif
 
@@ -241,76 +238,33 @@ wave_out_volume(uint16 left, uint16 right)
 	pv[0].sizeIn = 8;
 	if (alSetParams(AL_DEFAULT_OUTPUT, pv, 1) < 0)
 	{
-		fprintf(stderr, "wave_out_volume: alSetParams failed: %s\n",
+		fprintf(stderr, "sgi_volume: alSetParams failed: %s\n",
 			alGetErrorString(oserror()));
 		return;
 	}
 
 #if (defined(IRIX_DEBUG))
-	fprintf(stderr, "wave_out_volume: returning\n");
+	fprintf(stderr, "sgi_volume: returning\n");
 #endif
 }
 
 void
-wave_out_write(STREAM s, uint16 tick, uint8 index)
-{
-	struct audio_packet *packet = &packet_queue[queue_hi];
-	unsigned int next_hi = (queue_hi + 1) % MAX_QUEUE;
-
-	if (next_hi == queue_lo)
-	{
-		fprintf(stderr, "No space to queue audio packet\n");
-		return;
-	}
-
-	queue_hi = next_hi;
-
-	packet->s = *s;
-	packet->tick = tick;
-	packet->index = index;
-	packet->s.p += 4;
-
-	/* we steal the data buffer from s, give it a new one */
-	s->data = malloc(s->size);
-
-	if (!g_dsp_busy)
-		wave_out_play();
-}
-
-void
-wave_out_play(void)
+sgi_play(void)
 {
 	struct audio_packet *packet;
 	ssize_t len;
 	unsigned int i;
-	uint8 swap;
 	STREAM out;
-	static BOOL swapped = False;
 	int gf;
 
 	while (1)
 	{
-		if (queue_lo == queue_hi)
-		{
-			g_dsp_busy = False;
+		if (rdpsnd_queue_empty())
 			return;
-		}
 
-		packet = &packet_queue[queue_lo];
+		packet = rdpsnd_queue_current_packet();
 		out = &packet->s;
 
-		/* Swap the current packet, but only once */
-		if (g_swapaudio && !swapped)
-		{
-			for (i = 0; i < out->end - out->p; i += 2)
-			{
-				swap = *(out->p + i);
-				*(out->p + i) = *(out->p + i + 1);
-				*(out->p + i + 1) = swap;
-			}
-			swapped = True;
-		}
-
 		len = out->end - out->p;
 
 		alWriteFrames(output_port, out->p, len / combinedFrameSize);
@@ -319,21 +273,47 @@ wave_out_play(void)
 		if (out->p == out->end)
 		{
 			gf = alGetFilled(output_port);
-			if ( gf < (4 * maxFillable / 10) )
+			if (gf < (4 * maxFillable / 10))
 			{
-				rdpsnd_send_completion(packet->tick, packet->index);
-				free(out->data);
-				queue_lo = (queue_lo + 1) % MAX_QUEUE;
-				swapped = False;
+				rdpsnd_queue_next(0);
 			}
 			else
 			{
 #if (defined(IRIX_DEBUG))
 /*  				fprintf(stderr,"Busy playing...\n"); */
 #endif
-				g_dsp_busy = True;
+				usleep(10);
 				return;
 			}
 		}
 	}
 }
+
+struct audio_driver *
+sgi_register(char *options)
+{
+	static struct audio_driver sgi_driver;
+
+	memset(&sgi_driver, 0, sizeof(sgi_driver));
+
+	sgi_driver.name = "sgi";
+	sgi_driver.description = "SGI output driver";
+
+	sgi_driver.add_fds = sgi_add_fds;
+	sgi_driver.check_fds = sgi_check_fds;
+
+	sgi_driver.wave_out_open = sgi_open;
+	sgi_driver.wave_out_close = sgi_close;
+	sgi_driver.wave_out_format_supported = sgi_format_supported;
+	sgi_driver.wave_out_set_format = sgi_set_format;
+	sgi_driver.wave_out_volume = sgi_volume;
+
+	sgi_driver.need_byteswap_on_be = 1;
+	sgi_driver.need_resampling = 0;
+
+	if (options)
+	{
+		sgi_output_device = xstrdup(options);
+	}
+	return &sgi_driver;
+}
